'''
% Normalized Chow Test
% This program conducts chow test with normalized log-cumulative hazard data.
%
% This program requires R, rpy, numpy, and chow test program.
%
%    R ==> http://www.r-project.org/
%    rpy ==> http://rpy.sourceforge.net/
%    numpy ==> http://numpy.scipy.org/
%
%    chow.py, lm.py, matlib.py functions were implemented by Dr. Ernesto P. Adorio in http://adorio-research.org/
%
%    chow.py ==> http://adorio-research.org/wordpress/?p=1789
%    lm.py ==> http://adorio-research.org/wordpress/?p=1932
%    matlib.py ==> http://adorio-research.org/wordpress/?p=1930
%
% Last Update: 2011.06.03
% By Jae-Seong Yang, destine@postech.ac.kr
'''

from chow import *
import numpy
import SurvivalData

def normalized_chow_test( x1, y1, x2, y2, alpha = 0.05 ):
   X = []
   Y = []
   mean_x1 = numpy.mean(x1)
   mean_y1 = numpy.mean(y1)
   for (i,w) in enumerate(x1):
      X.append([1.0, w - mean_x1])
      Y.append(y1[i] - mean_y1)
   breakpoint = i + 1

   mean_x2 = numpy.mean(x2)
   mean_y2 = numpy.mean(y2)
   for (i,w) in enumerate(x2):
      X.append([1.0, w - mean_x2])
      Y.append(y2[i] - mean_y2)
    
   return chow( X, Y, breakpoint, alpha )

def NormalizedChowTest( aData1, aData2, results_var3Test ):
    return normalized_chow_test( aData1.timelist_mot, aData1.mortality, aData2.timelist_mot, aData2.mortality )

def RunNormalizedChowTestAll( conditions ):
    results_var3Test = []

    for i in range( 0, len( conditions ) ):
        aData1 = conditions[i]
        for j in range( 0, len( conditions ) ):
            if i == j:
                continue
            aData2 = conditions[j]
            
            P_value, F_value, Fcrit, df1, df2, RSS, RSS1, RSS2 = NormalizedChowTest( aData1, aData2, results_var3Test )
            Corrected_P_value = min( P_value * ( len( conditions ) - 1 ), 1.0 )

            results_var3Test.append( [ aData1.name, aData2.name, F_value, df1, df2, P_value, Corrected_P_value ] )
            
    return results_var3Test


if __name__ == "__main__":    
    conditions = SurvivalData.ReadDataFromFile( "sample.txt" )
    SurvivalData.CalculateSubplotData( conditions )

    results_var3Test = RunNormalizedChowTestAll( conditions )

    print "\t".join( [ "Sample 1", "Sample 2", "F-value", "DF1", "DF2", "P-value", "Corrected P-value" ] )
    for results in results_var3Test:
        for v in results:
            print v, "\t",
        print ""

        
